@@ -6,13 +6,13 @@
 #
 
 `addKST` <-
-function (n = c(10, 10, 10, 15), nROC = c(10, 15, 20, 30), nSig = 9,
-    maType, wts = 1:NROW(n), ..., on = NA, legend = "auto")
+function (n = c(10, 10, 10, 15), nROC = c(10, 15, 20, 30), nSig = 9, 
+    maType, wts = 1:NROW(n), ..., on = NA, legend = "auto") 
 {
     lchob <- get.current.chob()
     x <- as.matrix(lchob@xdata)
     x <- coredata(Cl(x))
-    x <- KST(price = x, n = n, nROC = nROC, nSig = nSig, maType = maType,
+    x <- KST(price = x, n = n, nROC = nROC, nSig = nSig, maType = maType, 
         wts = wts)
     yrange <- NULL
     chobTA <- new("chobTA")
@@ -30,20 +30,21 @@ function (n = c(10, 10, 10, 15), nROC = c(10, 15, 20, 30), nSig = 9,
     }
     chobTA@call <- match.call()
     legend.name <- gsub("^addKST", "Know Sure Thing ", deparse(match.call()))
-    gpars <- c(list(...), list(col = 6:7))[unique(names(c(list(col = 6:7),
+    gpars <- c(list(...), list(col = 6:7))[unique(names(c(list(col = 6:7), 
         list(...))))]
-    chobTA@params <- list(xrange = lchob@xrange, yrange = yrange,
-        colors = lchob@colors, color.vol = lchob@color.vol, multi.col = lchob@multi.col,
-        spacing = lchob@spacing, width = lchob@width, bp = lchob@bp,
-        x.labels = lchob@x.labels, time.scale = lchob@time.scale,
-        isLogical = is.logical(x), legend = legend, legend.name = legend.name,
+    chobTA@params <- list(xrange = lchob@xrange, yrange = yrange, 
+        colors = lchob@colors, color.vol = lchob@color.vol, multi.col = lchob@multi.col, 
+        spacing = lchob@spacing, width = lchob@width, bp = lchob@bp, 
+        x.labels = lchob@x.labels, time.scale = lchob@time.scale, 
+        isLogical = is.logical(x), legend = legend, legend.name = legend.name, 
         pars = list(gpars))
     if (is.null(sys.call(-1))) {
         TA <- lchob@passed.args$TA
         lchob@passed.args$TA <- c(TA, chobTA)
-        lchob@windows <- lchob@windows + ifelse(chobTA@new, 1,
+        lchob@windows <- lchob@windows + ifelse(chobTA@new, 1, 
             0)
-        do.call(chartSeries.chob, list(lchob))
+        chartSeries.chob <- chartSeries.chob
+        do.call("chartSeries.chob", list(lchob))
         invisible(chobTA)
     }
     else {
