@@ -10,28 +10,24 @@ function(Symbol,from='1970-01-01',to=Sys.Date(),env=parent.frame(),src='yahoo',
                         deparse(substitute(Symbol)),
                         as.character(Symbol))
 
-  yahoo.URL <- 'https://ichart.finance.yahoo.com/table.csv?s='
-  from.y <- as.numeric(strsplit(as.character(from), "-", )[[1]][1])
-  from.m <- as.numeric(strsplit(as.character(from), "-", )[[1]][2])-1
-  from.d <- as.numeric(strsplit(as.character(from), "-", )[[1]][3])
-  to.y <- as.numeric(strsplit(as.character(to), "-", )[[1]][1])
-  to.m <- as.numeric(strsplit(as.character(to), "-", )[[1]][2])-1
-  to.d <- as.numeric(strsplit(as.character(to), "-", )[[1]][3])
+  from.posix <- .dateToUNIX(from)
+  to.posix <- .dateToUNIX(to)
 
   tmp <- tempfile()
   on.exit(unlink(tmp))
-  download.file(paste(yahoo.URL,Symbol.name, "&a=", 
-            from.m, "&b=", sprintf("%.2d", from.d), "&c=", from.y, 
-            "&d=", to.m, "&e=", sprintf("%.2d", to.d), "&f=", 
-            to.y, "&g=v&ignore=.csv", 
-            sep = ""), destfile = tmp, quiet = !verbose)
+
+  handle <- .getHandle()
+  yahoo.URL <- .yahooURL(Symbol.name, from.posix, to.posix,
+                         "1d", "div", handle)
+  curl::curl_download(yahoo.URL, destfile=tmp, quiet=!verbose, handle=handle$ch)
+
   fr <- read.csv(tmp)
   fr <- xts(fr[,2],as.Date(fr[,1]))
   colnames(fr) <- paste(Symbol.name,'div',sep='.')
 
   # dividends from Yahoo are split-adjusted; need to un-adjust
   if(src[1] == "yahoo" && !split.adjust) {
-    splits <- getSplits(Symbol.name, from="1900-01-01")
+    splits <- getSplits(Symbol.name, from="1949-01-01")
     if(is.xts(splits) && is.xts(fr) && nrow(splits) > 0 && nrow(fr) > 0) {
       fr <- fr / adjRatios(splits=merge(splits, index(fr)))[,1]
     }
